﻿using System.Net.Http.Json;

using Newtonsoft.Json;

namespace TestClient
{
    /// <summary>
    /// 부품 관리자
    /// </summary>
    public static class PartManager
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// HTTP 클라이언트
        /// </summary>
        private static HttpClient _client;

        /// <summary>
        /// 기본 주소
        /// </summary>
        private static readonly string _baseAddress = "http://icodebroker.asuscomm.com:5210";

        /// <summary>
        /// URL
        /// </summary>
        private static readonly string _url = $"{_baseAddress}/api/";

        /// <summary>
        /// 권한 키
        /// </summary>
        private static string _authorizationKey;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 부품 열거 가능형 구하기 (비동기) - GetPartEnumerableAsync()

        /// <summary>
        /// 부품 열거 가능형 구하기 (비동기)
        /// </summary>
        /// <returns>부품 열거 가능형 태스크</returns>
        public static async Task<IEnumerable<Part>> GetPartEnumerableAsync()
        {
            if(Connectivity.Current.NetworkAccess != NetworkAccess.Internet)
            {
                return new List<Part>();
            }

            HttpClient client = await GetClient();

            string result = await client.GetStringAsync($"{_url}part");

            return JsonConvert.DeserializeObject<List<Part>>(result);  
        }

        #endregion

        #region 추가하기 (비동기) - AddAsync(partName, partType, supplier)

        /// <summary>
        /// 추가하기 (비동기)
        /// </summary>
        /// <param name="partName">부품명</param>
        /// <param name="partType">부품 타입</param>
        /// <param name="supplier">공급자</param>
        /// <returns>부품 태스크</returns>
        public static async Task<Part> AddAsync(string partName, string partType, string supplier)
        {
            if(Connectivity.Current.NetworkAccess != NetworkAccess.Internet)
            {
                return new Part();
            }

            Part part = new Part()
            {
                PartName          = partName,
                SupplierList      = new List<string>(new[] { supplier }),
                PartID            = string.Empty,
                PartType          = partType,
                PartAvailableDate = DateTime.Now.Date
            };

            HttpRequestMessage requestMessage = new HttpRequestMessage(HttpMethod.Post, $"{_url}part");

            requestMessage.Content = JsonContent.Create<Part>(part);

            HttpResponseMessage responseMessage = await _client.SendAsync(requestMessage);

            responseMessage.EnsureSuccessStatusCode();

            string json = await responseMessage.Content.ReadAsStringAsync();

            Part partInserted = JsonConvert.DeserializeObject<Part>(json);

            return partInserted;
        }

        #endregion
        #region 수정하기 비동기) - UpdateAsync(part)

        /// <summary>
        /// 수정하기 비동기)
        /// </summary>
        /// <param name="part">부품</param>
        /// <returns>태스크</returns>
        public static async Task UpdateAsync(Part part)
        {
            if(Connectivity.Current.NetworkAccess != NetworkAccess.Internet)
            {
                return;
            }

            HttpRequestMessage requestMessage = new(HttpMethod.Put, $"{_url}part/{part.PartID}");

            requestMessage.Content = JsonContent.Create<Part>(part);

            HttpClient client = await GetClient();

            HttpResponseMessage responseMessage = await client.SendAsync(requestMessage);

            responseMessage.EnsureSuccessStatusCode();
        }

        #endregion
        #region 삭제하기 (비동기) - DeleteAsync(partID)

        /// <summary>
        /// 삭제하기 (비동기)
        /// </summary>
        /// <param name="partID">부품 ID</param>
        /// <returns>태스크</returns>
        public static async Task DeleteAsync(string partID)
        {
            if(Connectivity.Current.NetworkAccess != NetworkAccess.Internet)
            {
                return;
            }

            HttpRequestMessage msg = new(HttpMethod.Delete, $"{_url}part/{partID}");

            HttpClient client = await GetClient();

            var response = await client.SendAsync(msg);

            response.EnsureSuccessStatusCode();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region HTTP 클라이언트 구하기 - GetClient()

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <returns>HTTP 클라이언트</returns>
        private static async Task<HttpClient> GetClient()
        {
            if(_client != null)
            {
                return _client;
            }

            _client = new HttpClient();

            if(string.IsNullOrEmpty(_authorizationKey))
            {                
                _authorizationKey = await _client.GetStringAsync($"{_url}login");
                _authorizationKey = JsonConvert.DeserializeObject<string>(_authorizationKey);
            }

            _client.DefaultRequestHeaders.Add("Authorization", _authorizationKey );
            _client.DefaultRequestHeaders.Add("Accept"       , "application/json");

            return _client;
        }

        #endregion
    }
}