﻿using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Input;

namespace TestClient;

/// <summary>
/// 부품 뷰 모델
/// </summary>
public class PartViewModel : INotifyPropertyChanged
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Event
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 속성 변경시 - PropertyChanged

    /// <summary>
    /// 속성 변경시
    /// </summary>
    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 갱신 여부
    /// </summary>
    private bool isRefreshing = false;

    /// <summary>
    /// 실행 여부
    /// </summary>
    private bool isBusy = false;

    /// <summary>
    /// 부품 컬렉션
    /// </summary>
    private ObservableCollection<Part> partCollection;

    /// <summary>
    /// 선택 부품
    /// </summary>
    private Part selectedPart;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 갱신 여부 - IsRefreshing

    /// <summary>
    /// 갱신 여부
    /// </summary>
    public bool IsRefreshing
    {
        get => this.isRefreshing;
        set
        {
            if(this.isRefreshing == value)
            {
                return;
            }

            this.isRefreshing = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsRefreshing)));
        }
    }

    #endregion
    #region 실행 여부 - IsBusy

    /// <summary>
    /// 실행 여부
    /// </summary>
    public bool IsBusy
    {
        get => this.isBusy;
        set
        {
            if(this.isBusy == value)
            {
                return;
            }

            this.isBusy = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(IsBusy)));
        }
    }

    #endregion
    #region 부품 컬렉션 - PartCollection

    /// <summary>
    /// 부품 컬렉션
    /// </summary>
    public ObservableCollection<Part> PartCollection
    {
        get => this.partCollection;
        set => this.partCollection = value;
    }

    #endregion
    #region 선택 부품 - SelectedPart

    /// <summary>
    /// 선택 부품
    /// </summary>
    public Part SelectedPart
    {
        get => this.selectedPart;
        set
        {
            if(this.selectedPart == value)
            {
                return;
            }

            this.selectedPart = value;

            PropertyChanged.Invoke(this, new PropertyChangedEventArgs(nameof(SelectedPart)));
        }
    }

    #endregion

    #region 데이터 로드 명령 - LoadDataCommand

    /// <summary>
    /// 데이터 로드 명령
    /// </summary>
    public ICommand LoadDataCommand { get; private set; }

    #endregion
    #region 부품 선택시 명령 - PartSelectedCommand

    /// <summary>
    /// 부품 선택시 명령
    /// </summary>
    public ICommand PartSelectedCommand { get; private set; }

    #endregion
    #region 신규 부품 추가 명령 - AddNewPartCommand

    /// <summary>
    /// 신규 부품 추가 명령
    /// </summary>
    public ICommand AddNewPartCommand { get; private set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - PartViewModel()

    /// <summary>
    /// 생성자
    /// </summary>
    public PartViewModel()
    {            
        this.partCollection = new ObservableCollection<Part>();

        LoadDataCommand     = new Command(async () => await ProcessLoadData());
        PartSelectedCommand = new Command(async () => await ProcessPartSelected());
        AddNewPartCommand   = new Command(async () => await Shell.Current.GoToAsync("addpart"));

        MessagingCenter.Subscribe<AddPartViewModel>(this, "refresh", async (sender) => await ProcessLoadData());

        Task.Run(ProcessLoadData);
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 데이터 로드 처리하기 - ProcessLoadData()

    /// <summary>
    /// 데이터 로드 처리하기
    /// </summary>
    /// <returns>태스크</returns>
    private async Task ProcessLoadData()
    {
        if(IsBusy)
        {
            return;
        }

        try
        {
            IsRefreshing = true;
            IsBusy       = true;

            IEnumerable<Part> partEnumerable = await PartManager.GetPartEnumerableAsync();

            MainThread.BeginInvokeOnMainThread
            (
                () =>
                {
                    PartCollection.Clear();
                
                    foreach(Part part in partEnumerable)
                    {
                        PartCollection.Add(part);
                    }
                }
            );
        }
        finally
        {    
            IsRefreshing = false;
            IsBusy       = false;
        }
    }

    #endregion
    #region 부품 선택시 처리하기 - ProcessPartSelected()

    /// <summary>
    /// 부품 선택시 처리하기
    /// </summary>
    /// <returns>태스크</returns>
    private async Task ProcessPartSelected()
    {
        if(SelectedPart == null)
        {
            return;
        }

        Dictionary<string, object> parameterDictionary = new Dictionary<string, object>()
        {
            { "part", SelectedPart }
        };

        await Shell.Current.GoToAsync("addpart", parameterDictionary);

        MainThread.BeginInvokeOnMainThread(() => SelectedPart = null);
    }

    #endregion
}