﻿using System.Net;
using System.Text.Json;

using Microsoft.AspNetCore.Mvc;

using TestServer.Models;

namespace TestServer.Controllers;

/// <summary>
/// 부품 컨트롤러
/// </summary>
[Route("api/[controller]")]
[ApiController]
public class PartController : BaseController
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 조회하기 - Get()

    /// <summary>
    /// 조회하기
    /// </summary>
    /// <returns>액션 결과</returns>
    [HttpGet]
    public ActionResult Get()
    {
        bool authorized = CheckAuthorization();

        if(!authorized)
        {
            return Unauthorized();
        }

        Console.WriteLine("GET /api/part");

        return new JsonResult(UserPartList);
    }

    #endregion
    #region 조회하기 - Get(partID)

    /// <summary>
    /// 조회하기
    /// </summary>
    /// <param name="partID">부품 ID</param>
    /// <returns>액션 결과</returns>
    [HttpGet("{partid}")]
    public ActionResult Get(string partID)
    {
        bool authorized = CheckAuthorization();

        if(!authorized)
        {
            return Unauthorized();
        }

        if(string.IsNullOrEmpty(partID))
        {
            return BadRequest();
        }

        partID = partID.ToUpperInvariant();

        Console.WriteLine($"GET /api/part/{partID}");

        List<Part> userPartList = UserPartList;

        Part part = userPartList.SingleOrDefault(x => x.PartID == partID);

        if(part == null)
        {
            return NotFound();
        }
        else
        {
            return Ok(part);
        }
    }

    #endregion

    #region 추가하기 - Post(part)

    /// <summary>
    /// 추가하기
    /// </summary>
    /// <param name="part">부품</param>
    /// <returns>액션 결과</returns>
    [HttpPost]
    public ActionResult Post([FromBody] Part part)
    {
        try
        {
            bool authorized = CheckAuthorization();

            if(!authorized)
            {
                return Unauthorized();
            }

            if(!string.IsNullOrWhiteSpace(part.PartID))
            {
                return BadRequest();
            }

            Console.WriteLine($"POST /api/part");

            Console.WriteLine(JsonSerializer.Serialize(part));

            part.PartID = PartFactory.CreatePartID();

            if(!ModelState.IsValid)
            {
                return BadRequest();
            }

            List<Part> userPartList = UserPartList;

            if(userPartList.Any(x => x.PartID == part.PartID))
            {
                return this.Conflict();
            }

            userPartList.Add(part);

            return this.Ok(part);
        }
        catch(Exception)
        {
            return this.Problem("Internal server error");
        }
    }

    #endregion
    #region 수정하기 - Put(partID, part)

    /// <summary>
    /// 수정하기
    /// </summary>
    /// <param name="partID">부품 ID</param>
    /// <param name="part">부품</param>
    /// <returns>응답 메시지</returns>
    [HttpPut("{partid}")]
    public HttpResponseMessage Put(string partID, [FromBody] Part part)
    {
        try
        {
            bool authorized = CheckAuthorization();

            if(!authorized)
            {
                return new HttpResponseMessage(HttpStatusCode.Unauthorized);
            }

            if (!ModelState.IsValid)
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }

            if(string.IsNullOrEmpty(part.PartID))
            {
                return new HttpResponseMessage(HttpStatusCode.BadRequest);
            }

            Console.WriteLine($"PUT /api/part/{partID}");

            Console.WriteLine(JsonSerializer.Serialize(part));

            List<Part> userPartList = UserPartList;

            Part existingPart = userPartList.SingleOrDefault(x => x.PartID == partID);

            if(existingPart != null)
            {
                existingPart.SupplierList      = part.SupplierList;
                existingPart.PartType          = part.PartType;
                existingPart.PartAvailableDate = part.PartAvailableDate;
                existingPart.PartName          = part.PartName;
            }

            return new HttpResponseMessage(HttpStatusCode.OK);
        }
        catch(Exception)
        {
            return new HttpResponseMessage(HttpStatusCode.InternalServerError);
        }
    }

    #endregion
    #region 삭제하기 - Delete(partID)

    /// <summary>
    /// 삭제하기
    /// </summary>
    /// <param name="partID">부품 ID</param>
    /// <returns>응답 메시지</returns>
    [HttpDelete]
    [Route("{partid}")]
    public HttpResponseMessage Delete(string partID)
    {
        try
        {
            bool authorized = CheckAuthorization();

            if(!authorized)
            {
                return new HttpResponseMessage(HttpStatusCode.Unauthorized);
            }

            List<Part> userPartList = UserPartList;

            Part existingPart = userPartList.SingleOrDefault(x => x.PartID == partID);

            if(existingPart == null)
            {
                return new HttpResponseMessage(HttpStatusCode.NotFound);
            }

            Console.WriteLine($"POST /api/part/{partID}");

            userPartList.RemoveAll(x => x.PartID == partID);

            return new HttpResponseMessage(HttpStatusCode.OK);
        }
        catch(Exception)
        {
            return new HttpResponseMessage(HttpStatusCode.InternalServerError);
        }
    }

    #endregion
}